-- -*- coding: utf-8 -*-

local keywords = Set(explode(' ',props['keywords.$(file.patterns.lithaskell)']))

function LitHaskellLexer(styler)
  local S_DEFAULT = 0
  local S_IDENTIFIER = 1
  local S_KEYWORD = 2
  local S_NUMBER = 3
  local S_STRING = 4
  local S_CHAR = 5
  local S_CAPITAL = 8
  local S_DATA = 9
  local S_OPERATOR = 11
  local S_LINE_COMMENT = 13
  local S_BLOCK_COMMENT = 14
  local S_ERROR = 17

  local M_DEFAULT = 0
  local M_DATA = 1
  local mode = M_DEFAULT

  local isInBinOpFunc = false

  local upperCaseChars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
  local lowerCaseChars = "_abcdefghijklmnopqrstuvwxyz"
  local identStartChars = upperCaseChars .. lowerCaseChars
  local numStartChars = "0123456789"
  local identChars = identStartChars .. numStartChars .. "\'"
  local numChars = numStartChars .. "."
  local operators = "+-*:=?/#~<&>.,;|()[]{}@"

  local isInCodeRegion = false
  local isError = false
  styler:StartStyling(styler.startPos, styler.lengthDoc, styler.initStyle)
  while styler:More() do
    if styler:AtLineStart() then
      if styler:State() == S_LINE_COMMENT then
        styler:SetState(S_DEFAULT)
      end
      local currentLine = editor:GetLine(styler:Line(styler:Position()))
      if currentLine ~= nil and currentLine:match('%S') then
        local prevLine = editor:GetLine(styler:Line(styler:Position()) - 1)
        local prevCodeRegion = prevLine ~= nil and prevLine:sub(1,1) == ">" or isInCodeRegion

        isInCodeRegion = styler:Current() == ">"
        -- error happens when program line appears directly after or before comment
        -- an empty line between them is required as mandated by unlit
        isError = prevCodeRegion ~= isInCodeRegion and prevLine ~= nil and not prevLine:match('^%s*$')
        styler:SetState(isError and S_ERROR or S_DEFAULT)
      end
    end

    if not isError and isInCodeRegion then
      if styler:State() == S_IDENTIFIER then
        if not identChars:find(styler:Current(), 1, true) then
          local identifier = styler:Token()
          if keywords[identifier] or upperCaseChars:find(identifier:sub(1,1),1,true) then
            styler:ChangeState(mode == M_DATA and S_DATA or S_KEYWORD)
            mode = identifier == "data" and M_DATA or M_DEFAULT
          end
          styler:SetState(S_DEFAULT)
        end
      elseif styler:State() == S_NUMBER then
        if not numChars:find(styler:Current(), 1, true) then
          styler:SetState(S_DEFAULT)
        end
      elseif styler:State() == S_OPERATOR then
        if not isInBinOpFunc then
          if not operators:find(styler:Current(), 1, true) then
            styler:SetState(S_DEFAULT)
          end
        else
        if styler:Current() == "`" then
          styler:ForwardSetState(S_DEFAULT)
          isInBinOpFunc = false
        end
      end
    elseif styler:State() == S_CHAR then
      if styler:Current() == "\'" then
        styler:ForwardSetState(S_DEFAULT)
      end
    elseif styler:State() == S_STRING then
      if styler:Current() == '"' then
        styler:ForwardSetState(S_DEFAULT)
      end
    elseif styler:State() == S_BLOCK_COMMENT then
      if styler:Current() == '-' and styler:Next() == '}' then
        styler:Forward()
        styler:ForwardSetState(S_DEFAULT)
      end
    end

    if styler:State() == S_DEFAULT then
      if identStartChars:find(styler:Current(), 1, true) then
        styler:SetState(S_IDENTIFIER)
      elseif numStartChars:find(styler:Current(), 1, true) then
        styler:SetState(S_NUMBER)
      elseif styler:Current() == '-' and styler:Next() == '-' then
        styler:SetState(S_LINE_COMMENT)
      elseif styler:Current() == '{' and styler:Next() == '-' then
        styler:SetState(S_BLOCK_COMMENT)
      elseif operators:find(styler:Current(), 1, true) then
        styler:SetState(S_OPERATOR)
      elseif styler:Current() == "\'" then
        styler:SetState(S_CHAR)
      elseif styler:Current() == '"' then
        styler:SetState(S_STRING)
      elseif styler:Current() == '`' then
        styler:SetState(S_OPERATOR)
        isInBinOpFunc = true
      end
    end
  end
  styler:Forward()
end
styler:EndStyling()
end

return LitHaskellLexer
