-- -*- coding: utf-8 -*-

local keywords = Set(explode(' ',props['keywords.$(file.patterns.oberon)']))

function OberonLexer(styler)
  local S_DEFAULT = 0
  local S_IDENTIFIER = 1
  local S_KEYWORD = 2
  local S_NUMBER = 3
  local S_OPERATOR = 4
  local S_STRINGSGL = 5
  local S_STRINGDBL = 6
  local S_COMMENT = 7

  local identStartChars = "_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
  local numStartChars = "0123456789"
  local identChars = identStartChars .. numStartChars
  local numChars = numStartChars .. "."
  local operators = "+-*:=?/#~<&>.,;|()[]{}"

  styler:StartStyling(styler.startPos, styler.lengthDoc, styler.initStyle)
  while styler:More() do

    if styler:State() == S_IDENTIFIER then
      if not identChars:find(styler:Current(), 1, true) then
        local styler_orig_pos = styler:Position()
        local identifier = styler:Token()
        if keywords[identifier] then
          -- attempt to do automatic capitalization
          -- local original_pos = editor.CurrentPos
          -- editor:SetSel(styler_orig_pos - identifier:len(),styler_orig_pos)
          -- editor:ReplaceSel(identifier:upper())
          -- editor:SetSel(original_pos,original_pos)
          styler:ChangeState(S_KEYWORD)
        end
        styler:SetState(S_DEFAULT)
      end
    elseif styler:State() == S_NUMBER then
      if not numChars:find(styler:Current(), 1, true) then
        styler:SetState(S_DEFAULT)
      end
    elseif styler:State() == S_OPERATOR then
      if not operators:find(styler:Current(), 1, true) then
        styler:SetState(S_DEFAULT)
      end
    elseif styler:State() == S_STRINGSGL then
      if styler:Current() == "\'" then
        styler:ForwardSetState(S_DEFAULT)
      end
    elseif styler:State() == S_STRINGDBL then
      if styler:Current() == '"' then
        styler:ForwardSetState(S_DEFAULT)
      end
    elseif styler:State() == S_COMMENT then
      if styler:Current() == '*' and styler:Next() == ')' then
        styler:Forward()
        styler:ForwardSetState(S_DEFAULT)
      end
    end

    if styler:State() == S_DEFAULT then
      if identStartChars:find(styler:Current(), 1, true) then
        styler:SetState(S_IDENTIFIER)
      elseif numStartChars:find(styler:Current(), 1, true) then
        styler:SetState(S_NUMBER)
      elseif styler:Current() == '(' and styler:Next() == '*' then
        styler:SetState(S_COMMENT)
      elseif operators:find(styler:Current(), 1, true) then
        styler:SetState(S_OPERATOR)
      elseif styler:Current() == "\'" then
        styler:SetState(S_STRINGSGL)
      elseif styler:Current() == '"' then
        styler:SetState(S_STRINGDBL)
      end
    end

    styler:Forward()
  end
  styler:EndStyling()
end

return OberonLexer
