-- -*- coding: utf-8 -*-

local keywords = Set(explode(' ',props['keywords.$(file.patterns.prolog)']))

function PrologLexer(styler)
  local S_DEFAULT = 0
  local S_QUOTED_ATOM = 1
  local S_RULE = 2
  local S_KEYWORD = 3
  local S_NUMBER = 4
  local S_OPERATOR = 5
  local S_VARIABLE = 6
  local S_LINE_COMMENT = 7
  local S_BLOCK_COMMENT = 8

  local upperCaseChars = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
  local lowerCaseChars = "_abcdefghijklmnopqrstuvwxyz"
  local identStartChars = upperCaseChars .. lowerCaseChars
  local numStartChars = "0123456789"
  local identChars = identStartChars .. numStartChars
  local numChars = numStartChars .. "."
  local operators = "!+-*:=?/\\#~<&>.,;|()[]{}@"

  styler:StartStyling(styler.startPos, styler.lengthDoc, styler.initStyle)
  while styler:More() do
    if styler:AtLineStart() then
      if styler:State() == S_LINE_COMMENT then
        styler:SetState(S_DEFAULT)
      end
    end

    if styler:State() == S_QUOTED_ATOM then
      if styler:Current() == "\'" then
        styler:ForwardSetState(S_DEFAULT)
      end
    elseif styler:State() == S_RULE then
      if not identChars:find(styler:Current(),1,true) then
        local identifier = styler:Token()
        if keywords[identifier] then
          styler:ChangeState(S_KEYWORD)
        end
        styler:SetState(S_DEFAULT)
      end
    elseif styler:State() == S_VARIABLE then
      if not identChars:find(styler:Current(),1,true) then
        styler:SetState(S_DEFAULT)
      end
    elseif styler:State() == S_NUMBER then
      if not numChars:find(styler:Current(),1,true) then
        styler:SetState(S_DEFAULT)
      end
    elseif styler:State() == S_OPERATOR then
      if not operators:find(styler:Current(),1,true) then
        styler:SetState(S_DEFAULT)
      end
    elseif styler:State() == S_BLOCK_COMMENT then
      if styler:Current() == '*' and styler:Next() == '/' then
        styler:Forward()
        styler:ForwardSetState(S_DEFAULT)
      end
    end

    if styler:State() == S_DEFAULT then
      if styler:Current() == "\'" then
        styler:SetState(S_QUOTED_ATOM)
      elseif upperCaseChars:find(styler:Current(),1,true) then
        styler:SetState(S_VARIABLE)
      elseif lowerCaseChars:find(styler:Current(),1,true) then
        styler:SetState(S_RULE)
      elseif numStartChars:find(styler:Current(),1,true) then
        styler:SetState(S_NUMBER)
      elseif styler:Current() == '%' then
        styler:SetState(S_LINE_COMMENT)
      elseif styler:Current() == '/' and styler:Next() == '*' then
        styler:SetState(S_BLOCK_COMMENT)
      elseif operators:find(styler:Current(),1,true) then
        styler:SetState(S_OPERATOR)
      end
    end

    styler:Forward()
  end
  styler:EndStyling()
end

return PrologLexer
