local in_word,current_word, substituting
local find = string.find

function indent(size)
  while size >= 0 do
    editor:AddText(" ")
    size = size - 1
  end
end

-- Expand "if" to
-- if () {
--
-- }
-- And set the cursor between the ()
-- Also works for "while "
function expandIf()
  -- The text is "if"
  editor:AddText("(")
  -- "if ("
  -- Remember where to bring the cursor to
  local tmp = editor.CurrentPos
  editor:AddText(") {")
  local line = editor:LineFromPosition(editor.CurrentPos)
  local tmpi = editor.LineIndentation[line]

  editor:AddText("\n")
  indent(tmpi + 1)
  editor:AddText("\n")
  tmpi = editor.LineIndentation[line]
  indent(tmpi - 1)
  editor:AddText("}")

  -- Bring the cursor into the "()"
  editor:GotoPos(tmp)
end

function expandFor()
  local line = editor:LineFromPosition(editor.CurrentPos)
  local tmpi = editor.LineIndentation[line]

  editor:AddText("(")
  local tmp = editor.CurrentPos
  editor:AddText(";;) {\n")
  indent(tmpi + 1)
  editor:AddText("\n")
  tmpi = editor.LineIndentation[line]
  indent(tmpi - 1)
  editor:AddText("}")
  editor:GotoPos(tmp)
end

function expandDo()
  local line = editor:LineFromPosition(editor.CurrentPos)
  local tmpi = editor.LineIndentation[line]

  editor:AddText("{\n")
  indent(tmpi + 1)
  local tmp = editor.CurrentPos
  editor:AddText("\n")
  tmpi = editor.LineIndentation[line]
  indent(tmpi - 1)
  editor:AddText("} while (")
  editor:AddText(");")
  editor:GotoPos(tmp)
end

function expandCout()
  editor:AddText("<< \"")
  local tmp = editor.CurrentPos
  editor:AddText("\" << endl;")
  editor:GotoPos(tmp)
end

local SubstitutingExt = {
  ['cpp']       = true,
  ['hypertext'] = true
}

function autoexpand(c)
  if not SubstitutingExt[editor:GetLexerLanguage()] then
    return false
  end

  -- Only activate on the space character
  if c == ' ' then
    -- The currentPos is the character after the cursor
    -- Get the style of the word that was just typed
    -- That is, the style of the character before the space
    cstyle = editor.StyleAt[editor.CurrentPos-2]

    -- If the typed word is a part of code
    -- Rather than a part of comments
    AcceptedCStyles = {
      [0]  = true,
      [32] = true,
      [4]  = true,
      [5]  = true,
      [10] = true,
      [11] = true,
      [16] = true
    }
    if AcceptedCStyles[cstyle] then

      -- Get the word that was just typed
      local p, original_pos
      local lineStart

      -- get the current position and the start of the current line
      lineStart = editor:PositionFromLine(editor:LineFromPosition(editor.CurrentPos))
      original_pos = editor.CurrentPos

      -- Find the beginning of the current word
      p = editor.CurrentPos-2
      -- Stop on the space/tab/newline character
      while (p >= lineStart) and (find(string.char(editor.CharAt[p]), '%w')) do
        p = p - 1
      end
      -- Increment 1 to get the first character of the current word
      p = p + 1

      -- Select the word and get it
      editor:SetSel(p, editor.CurrentPos-1)
      current_word = editor:GetSelText()
      -- Clear the selection
      editor:SetSel(original_pos, original_pos)
      -- Got the word, now expand
      -- if and while have the same expansion
      if current_word == "if" or current_word == "while" or current_word == "switch" then
        expandIf()
      elseif current_word == "for" then
        expandFor()
      elseif current_word == "do" then
        expandDo()
      elseif current_word == "cout" then
        expandCout()
      else -- nothing to expand, insert the new line
        return false
      end

      -- Expansion complete
      return true
    end
  end
end

scite_OnChar(autoexpand)
