program ValidatorDriver;

{$mode objfpc}{$H+}

uses
  Classes,SysUtils,Process;

type
  TDocType = (dtNone,dtXHTML,dtHTML,dtCSS);

var
  DocType: TDocType = dtNone;
  InputFile: String = '';
  CurrentDir: String;

function HandleOptions: Boolean;

  procedure HandleDocType;
  const
    ValidDocTypeCount = 3;
    ValidDocType: array [1..ValidDocTypeCount] of String = (
      'xhtml','html','css'
    );
  var
    i: Byte;
    Found: Boolean;
  begin
    i:=1;
    Found:=false;
    while not Found and (i<=ValidDocTypeCount) do
      if ParamStr(1)=ValidDocType[i] then begin
        Found:=true;
        DocType:=TDocType(i);
      end else Inc(i);
    if not Found then begin
      HandleOptions:=false;
      WriteLn('Invalid document type: '+ParamStr(1));
    end;
  end;

  procedure HandleInputFile;
  var
    f: Text;
  begin
    Assign(f,ParamStr(2));
    {$I-}
    Reset(f);
    {$I+}
    if IOResult<>0 then begin
      HandleOptions:=false;
      WriteLn('Cannot open input file: '+ParamStr(2));
    end else begin
      InputFile:=ParamStr(2);
    end;
    Close(f);
  end;

begin
  HandleOptions:=true;
  if ParamCount<>2 then
    HandleOptions:=false
  else begin
    HandleDocType;
    HandleInputFile;
  end;
end;

procedure CallValidator;
const
  ReadBytes = 2048;
var
  Buffer,Temp: String;
  BytesAvailable: LongWord;
  i,p: Integer;
begin
  with TProcess.Create(nil) do
    try
      case DocType of
        dtXHTML: CommandLine:=CurrentDir+'opensp\onsgmls -s -c '+CurrentDir+'dtds\xml.soc '+InputFile;
        dtHTML : CommandLine:=CurrentDir+'opensp\onsgmls -s -c '+CurrentDir+'dtds\sgml.soc '+InputFile;
        dtCSS  : CommandLine:='java -jar '+CurrentDir+'css.zip '+InputFile;
      end;
      Options:=[poUsePipes,poNoConsole,poStdErrToOutput];
      ShowWindow:=swoNone;
      try
        Execute;
        CurrentDirectory:=ExtractFilePath(ParamStr(0));
        Buffer:='';
        while Running do begin
          BytesAvailable:=Output.NumBytesAvailable;
          while BytesAvailable>0 do begin
            Temp:='';
            SetLength(Temp,BytesAvailable);
            Output.Read(Temp[1],BytesAvailable);
            Buffer:=Buffer+Temp;
            BytesAvailable:=Output.NumBytesAvailable;
          end;
        end;
        with TStringList.Create do
          try
            Text:=Buffer;
            if DocType<>dtCSS then
              for i:=0 to Count-1 do begin
                p:=Pos(InputFile,Strings[i]);
                WriteLn(Copy(Strings[i],p,Length(Strings[i])-p+1));
              end
            else
              for i:=0 to Count-1 do WriteLn(Strings[i]);
          finally
            Free;
          end;
      except
        on e: Exception do begin
          WriteLn(ErrOutput,e.Message);
          DumpExceptionBackTrace(ErrOutput);
        end;
      end;
    finally
      Free;
    end;
end;

procedure PrintHelp;
begin
  WriteLn('Usage: '+ExtractFileName(ParamStr(0))+' <xhtml|html|css> <input file>');
end;

begin
  CurrentDir:=ExtractFilePath(ParamStr(0));
  if HandleOptions then
    CallValidator
  else
    PrintHelp;
end.
